
/* Chrysalide - Outil d'analyse de fichiers binaires
 * ##FILE## - traduction d'instructions ARMv7
 *
 * Copyright (C) 2015 Cyrille Bagard
 *
 *  This file is part of Chrysalide.
 *
 *  Chrysalide is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Chrysalide is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */


@title SMMLA

@desc Signed Most Significant Word Multiply Accumulate multiplies two signed 32-bit values, extracts the most significant 32 bits of the result, and adds an accumulate value. Optionally, the instruction can specify that the result is rounded instead of being truncated. In this case, the constant 0x80000000 is added to the product before the high word is extracted.

@encoding (T1) {

	@word 1 1 1 1 1 0 1 1 0 1 0 1 Rn(4) Ra(4) Rd(4) 0 0 0 R(1) Rm(4)

	@syntax <reg_D> <reg_N> <reg_M> <reg_A>

	@conv {

		reg_D = Register(Rd)
		reg_N = Register(Rn)
		reg_M = Register(Rm)
		reg_A = Register(Ra)
		round = (R == '1')

	}

	@rules {

		if (round); chk_call ExtendKeyword("r")

	}

}

@encoding (A1) {

	@word cond(4) 0 1 1 1 0 1 0 1 Rd(4) Ra(4) Rm(4) 0 0 R(1) 1 Rn(4)

	@syntax <reg_D> <reg_N> <reg_M> <reg_A>

	@conv {

		reg_D = Register(Rd)
		reg_N = Register(Rn)
		reg_M = Register(Rm)
		reg_A = Register(Ra)
		round = (R == '1')

	}

	@rules {

		if (round); chk_call ExtendKeyword("r")
		chk_call StoreCondition(cond)

	}

}

